@@ -1,9 +1,9 @@
-#!/bin/sh
+#!/bin/bash
 #
 # Tested on several systems only
 # ttt1 Quite likely this needs changes to work with other distros and / or versions
 
-./AdjustMt.sh
+bash ./AdjustMt.sh
 #exit 1
 
 QMake=qmake
