@@ -90,6 +90,86 @@ tryPadding:
 }
 
 
+/*override*/ int Id3V240Frame::getOffset() const
+{
+    return 0 != (m_cFlag2 & 0x01) ? 4 : 0; // not quite OK if the flag is set but the frame is only several bytes, but that was invalid to begin with; anyway, it doesn't mess up loading from disk, since none is needed for such short frames
+}
+
+
+void Id3V240Frame::load(NoteColl& notes, istream& in, streampos posNext, bool bHasUnsynch)
+{
+    StreamStateRestorer rst (in);
+    if (m_nDiskDataSize < 0) { return; }
+    if (m_nDiskDataSize > 5000000) { return; } // there are probably no frames over 5MB
+
+    vector<char> v (m_nDiskDataSize);
+
+    m_vcData.clear();
+    //int nContentBytesSkipped (0);
+
+    int nRead (read(in, &v[0], m_nDiskDataSize));
+
+    //readID3V2(bHasUnsynch, in, &v[0], m_nMemDataSize1 w, posNext, nContentBytesSkipped));
+    //m_nDiskDataSize1 = q m_nMemDataSize1 + nContentBytesSkipped;
+    if (m_nDiskDataSize != nRead || !checkSize(in, posNext))
+    {
+        return;
+    }
+
+    rst.setOk();
+    if (bHasUnsynch && !v.empty())
+    {
+        for (int i = 0; i < cSize(v); ++i)
+        {
+            m_vcData.push_back(v[i]);
+            if (char(0xff) == v[i] && i < cSize(v) - 1 && 0 == v[i + 1]) // !!! it's OK to run this on the Data length indicator, because it doesn't contain any 0xff
+            {
+                ++i;
+            }
+        }
+    }
+    else
+    {
+        v.swap(m_vcData);
+    }
+
+    m_nMemDataSize = cSize(m_vcData);
+
+    if (getOffset() > 0 && m_nMemDataSize > 3)
+    { // Data length indicator
+        unsigned char* p (reinterpret_cast<unsigned char*> (&m_vcData[0]));
+        //inspect(p, 50);
+
+        int nDli ((p[0] << 21) + (p[1] << 14) + (p[2] << 7) + (p[3] << 0));
+
+        if (nDli != m_nMemDataSize - 4)
+        {
+            MP3_NOTE (m_pos, id3v240IncorrectDli); //ttt2 probably adjust m_nMemDataSize anyway; after all, the Data length indicator flag is still there
+        }
+        else
+        {
+            m_nMemDataSize -= 4;
+            m_vcData.erase(m_vcData.begin(), m_vcData.begin() + 4);
+        }
+    }
+}
+
+void Id3V240Frame::load(NoteColl& notes, istream& in, streampos posNext)
+{
+    load(notes, in, posNext, m_bHasUnsynch);
+    if (-1 == m_nMemDataSize)
+    {
+        load(notes, in, posNext, !m_bHasUnsynch);
+        if (-1 != m_nMemDataSize)
+        {
+            m_bHasUnsynch = !m_bHasUnsynch;
+            MP3_NOTE (m_pos, id3v240IncorrectFrameSynch);
+        }
+    }
+}
+
+
+
 Id3V240Frame::Id3V240Frame(NoteColl& notes, istream& in, streampos pos, bool bHasUnsynch, streampos posNext, StringWrp* pFileName) : Id3V2Frame(pos, bHasUnsynch, pFileName)
 {
     in.seekg(pos);
@@ -102,10 +182,11 @@ Id3V240Frame::Id3V240Frame(NoteColl& notes, istream& in, streampos pos, bool bHa
     if (0 == bfr[0])
     { // padding
         //m_nMemDataSize = -1; // !!! not needed; the constructor makes it -1
-        m_nDiskDataSize = -1;
+        //m_nDiskDataSize = -1; // !!! not needed; the constructor makes it -1
         m_szName[0] = 0;
         return;
     }
+    m_nDiskHdrSize = ID3_FRAME_HDR_SIZE + nHdrBytesSkipped;
 //inspect(bfr, ID3_FRAME_HDR_SIZE);
     strncpy(m_szName, bfr, 4);
     m_szName[4] = 0;
@@ -120,53 +201,36 @@ Id3V240Frame::Id3V240Frame(NoteColl& notes, istream& in, streampos pos, bool bHa
         }
     }
 
-    //MP3_CHECK (0 == (p[4] & 0x80) && 0 == (p[5] & 0x80) && 0 == (p[6] & 0x80) && 0 == (p[7] & 0x80), pos, "Invalid ID3V2.4.0 frame. Size is supposed to be synchsafe but it isn't.", Id3V240Stream::NotId3V2());
 
     m_cFlag1 = bfr[8];
     m_cFlag2 = bfr[9];
 //inspect(bfr);
 
-    MP3_CHECK (0 == (m_cFlag1 & ~0x60), pos, id3v2UnsuppFlags1, StreamIsUnsupported(Id3V240Stream::getClassDisplayName(), "ID3V2.4.0 tag containing a frame with an unsupported flag.")); // ignores "Tag alter preservation" and "File alter preservation" // ttt1 use them
-    MP3_CHECK (0 == (m_cFlag2 & ~0x02), pos, id3v2UnsuppFlags2, StreamIsUnsupported(Id3V240Stream::getClassDisplayName(), "ID3V2.4.0 tag containing a frame with an unsupported flag."));
-    m_bHasUnsynch = m_bHasUnsynch || (0 != (m_cFlag2 & ~0x02));
+    MP3_CHECK (0 == (m_cFlag1 & ~0x60), pos, id3v2UnsuppFlags1, StreamIsUnsupported(Id3V240Stream::getClassDisplayName(), "ID3V2.4.0 tag containing a frame with an unsupported flag.")); // ignores "Tag alter preservation" and "File alter preservation" // ttt2 use them
+    MP3_CHECK (0 == (m_cFlag2 & ~0x03), pos, id3v2UnsuppFlags2, StreamIsUnsupported(Id3V240Stream::getClassDisplayName(), "ID3V2.4.0 tag containing a frame with an unsupported flag."));
+
+    m_bHasUnsynch = (0 != (m_cFlag2 & ~0x02));
+//m_bHasUnsynch = false;
 
-    int nContentBytesSkipped (0);
-    nRead = 0;
 
     if (0 != (m_cFlag1 & 0x8f)) { MP3_NOTE (pos, id3v2IncorrectFlg1); }
     if (0 != (m_cFlag2 & 0xb0)) { MP3_NOTE (pos, id3v2IncorrectFlg2); }
 
-    //int nSize7Bit ((p[4] << 21) + (p[5] << 14) + (p[6] << 7) + (p[7] << 0));
+
     if (0 == (p[4] & 0x80) && 0 == (p[5] & 0x80) && 0 == (p[6] & 0x80) && 0 == (p[7] & 0x80))
-    { // could be 7 bit
-        m_nMemDataSize = (p[4] << 21) + (p[5] << 14) + (p[6] << 7) + (p[7] << 0);
-        m_vcData.resize(m_nMemDataSize);
-        streampos posUndo (in.tellg());
-        nRead = readID3V2(bHasUnsynch || (m_cFlag2 & 0x02), in, &m_vcData[0], m_nMemDataSize, posNext, nContentBytesSkipped); // ttt2 makes the assumption that unsynch is used if at least one of the global or frame-specific flag is set; not sure if it's right
-        m_nDiskHdrSize = ID3_FRAME_HDR_SIZE + nHdrBytesSkipped;
-        m_nDiskDataSize = m_nMemDataSize + nContentBytesSkipped;
-        if (m_nMemDataSize != nRead || !checkSize(in, posNext))
-        {
-            vector<char>().swap(m_vcData);;
-            m_nMemDataSize = -1;
-            in.seekg(posUndo);
-            //MP3_THROW (pos, "Invalid ID3V2.4.0 frame. File too short. Unable to read all the bytes declared in SIZE.", Id3V240Stream::NotId3V2());
-        }
+    { // by the specs it should be 7 bit unsynch
+        m_nDiskDataSize = (p[4] << 21) + (p[5] << 14) + (p[6] << 7) + (p[7] << 0);
+        load(notes, in, posNext);
     }
 
     if (-1 == m_nMemDataSize)
     { // failed to load as 7bit unsynch, so try 8bit
-        m_nMemDataSize = (p[4] << 24) + (p[5] << 16) + (p[6] << 8) + (p[7] << 0);
+        m_nDiskDataSize = (p[4] << 24) + (p[5] << 16) + (p[6] << 8) + (p[7] << 0);
+        load(notes, in, posNext);
 
-        m_vcData.resize(m_nMemDataSize);
-        nRead = readID3V2(bHasUnsynch || (m_cFlag2 & 0x02), in, &m_vcData[0], m_nMemDataSize, posNext, nContentBytesSkipped); // ttt2 makes the assumption that unsynch is used if at least one of the global or frame-specific flag is set; not sure if it's right
-        m_nDiskHdrSize = ID3_FRAME_HDR_SIZE + nHdrBytesSkipped;
-        m_nDiskDataSize = m_nMemDataSize + nContentBytesSkipped;
-        if (m_nMemDataSize != nRead || !checkSize(in, posNext))
+        if (-1 == m_nMemDataSize)
         {
-            vector<char>().swap(m_vcData);;
-            m_nMemDataSize = -1;
-            MP3_THROW (pos, id3v240FrameTooShort, StreamIsBroken(Id3V240Stream::getClassDisplayName(), "Truncated ID3V2.4.0 tag."));
+            MP3_THROW (pos, id3v240CantReadFrame, StreamIsBroken(Id3V240Stream::getClassDisplayName(), "Broken ID3V2.4.0 tag."));
         }
         else
         {
@@ -174,12 +238,34 @@ Id3V240Frame::Id3V240Frame(NoteColl& notes, istream& in, streampos pos, bool bHa
         }
     }
 
-
     try
     {
         try
         {
             getUtf8String();
+
+            if ('T' == m_szName[0])
+            {
+                if (!isTxxx() && 0 == m_nMemDataSize)
+                { // this is really invalid; text frames must have at least a byte; however, by doing these we make sure that an empty (i.e. having a single byte, for text encoding) frame gets copied if the tag is edited;
+                    m_vcData.clear();
+                    m_vcData.push_back(0);
+                    m_nMemDataSize = cSize(m_vcData);
+                    MP3_NOTE_D (pos, id3v2EmptyTextFrame, Notes::id3v2EmptyTextFrame().getDescription() + string(" (Frame:") + m_szName + ")");
+                }
+                else if (isTxxx() && m_nMemDataSize <= 1)
+                { // this is really invalid; text frames must have at least a byte; however, by doing these we make sure that an empty (i.e. having a single byte, for text encoding) frame gets copied if the tag is edited;
+                    m_vcData.clear();
+                    static const char* szInvalid ("INVALID");
+                    m_vcData.push_back(0); // latin1
+                    m_vcData.insert(m_vcData.end(), szInvalid, szInvalid + strlen(szInvalid)); // description
+                    m_vcData.push_back(0); // terminator
+                    m_vcData.insert(m_vcData.end(), szInvalid, szInvalid + strlen(szInvalid)); // value
+                    m_nMemDataSize = cSize(m_vcData);
+                    MP3_NOTE_D (pos, id3v2EmptyTextFrame, Notes::id3v2EmptyTextFrame().getDescription() + string(" (Frame:") + m_szName + ")"); // ttt2 actually TXXX is not text
+                }
+                //ttt2 add check for terminating 0 for TXXX, to split the value into descr and val
+            }
         }
         catch (const NotId3V2Frame&)
         {
@@ -203,9 +289,9 @@ Id3V240Frame::Id3V240Frame(NoteColl& notes, istream& in, streampos pos, bool bHa
 }
 
 
-// assumes Latin1, converts to UTF8; returns "<non-text value>" for non-text strings; unlike 2.3.0, which replaces '\0' with ' ' inside tags, this just considers a 0 to be a terminator; string fields in 2.4.0 are 0-terminated, while the ones in 2.3.0 aren't
-// whitespaces at the end of the string are removed; not sure if this is how it should be, though
-string Id3V240Frame::getUtf8String() const
+// may return multiple null characters; it's the job of getUtf8String() to deal with them;
+// chars after the first null are considered comments (or after the second null, for TXXX), so the nulls are replaced with commas, except for those at the end of the string (which are removed) and the first null in TXXX;
+string Id3V240Frame::getUtf8StringImpl() const
 {
     if ('T' != m_szName[0])
     {
@@ -219,16 +305,18 @@ string Id3V240Frame::getUtf8String() const
     // 2008.07.12 - on a second thought - throw but call this on the constructor, where it can be logged properly; if it worked on the constructor it should work later too
     CB_CHECK1 (m_nMemDataSize > 0, NotId3V2Frame());
     Id3V2FrameDataLoader wrp (*this);
-    const char* pData (wrp.getData());
+    const char* pData (wrp.getData()); //ttt2 from http://www.id3.org/id3v2.4.0-frames - All text information frames supports multiple strings, stored as a null separated list
+
+    string s;
+
     if (0 == pData[0])
     { // Latin-1
-        string s;
+
         for (int i = 1; i < m_nMemDataSize; ++i)
         {
             unsigned char c (pData[i]);
-            if (0 == c) { break; }
             if (c < 128)
-            {
+            { // !!! 0 is OK
                 s += char(c);
             }
             else
@@ -240,35 +328,51 @@ string Id3V240Frame::getUtf8String() const
                 s += char(c2);
             }
         }
-        rtrim(s);
-        return s;
     }
-
-    if (1 == pData[0])
+    else if (1 == pData[0])
     {
-        return utf8FromBomUtf16(pData + 1, m_nMemDataSize - 1);
+        s = utf8FromBomUtf16(pData + 1, m_nMemDataSize - 1);
     }
-
-    if (3 == pData[0])
+    else if (3 == pData[0])
     {
-        string s (pData + 1, m_nMemDataSize - 1);
-        if (!s.empty())
+        s = string(pData + 1, m_nMemDataSize - 1);
+    }
+    else
+    {
+        if (2 == pData[0])
         {
-            char c (s[s.size() - 1]);
-            if (0 == c) // this string is supposed to be 0-terminated; silently remove the ending null, if it's there; // ttt2 perhaps have warning, but only if somebody actually cares
+            CB_THROW1 (UnsupportedId3V2Frame()); //ttt2 add support for UTF-16BE (2 = "UTF-16BE [UTF-16] encoded Unicode [UNICODE] without BOM");
+        }
+
+        CB_THROW1 (NotId3V2Frame());
+    }
+
+    { // deal with nulls
+        int k (0);
+        if (isTxxx())
+        {
+            for (; k < cSize(s) && 0 != s[k]; ++k) {}
+            ++k;
+        }
+
+        int i (cSize(s) - 1);
+
+        for (; i >= k && 0 == s[i]; --i)
+        {
+            s.erase(i);
+        }
+
+        for (; i >= k; --i)
+        {
+            if (0 == s[i])
             {
-                s.erase(s.size() - 1);
+                //s.replace(i, 1, i > k ? ", " : "");
+                s.replace(i, 1, ", ");
             }
         }
-        return s;
     }
 
-    if (2 == pData[0])
-    {
-        CB_THROW1 (UnsupportedId3V2Frame()); //ttt1 add support for UTF-16BE and UTF-8 (2 = "UTF-16BE [UTF-16] encoded Unicode [UNICODE] without BOM");
-    }
-
-    CB_THROW1 (NotId3V2Frame());
+    return s;
 }
 
 
@@ -300,7 +404,7 @@ Id3V240Stream::Id3V240Stream(int nIndex, NoteColl& notes, istream& in, StringWrp
     MP3_CHECK_T (4 == bfr[3] && 0 == bfr[4], pos, "Invalid ID3V2.4.0 tag. Invalid ID3V2.4.0 header.", NotId3V2());
     m_nTotalSize = getId3V2Size (bfr);
     m_cFlags = bfr[5];
-    MP3_CHECK (0 == (m_cFlags & 0x7f), pos, id3v2UnsuppFlag, StreamIsUnsupported(Id3V240Stream::getClassDisplayName(), "ID3V2 tag with unsupported flag.")); //ttt1 review, support
+    MP3_CHECK (0 == (m_cFlags & 0x7f), pos, id3v2UnsuppFlag, StreamIsUnsupported(Id3V240Stream::getClassDisplayName(), "ID3V2 tag with unsupported flag.")); //ttt2 review, support
 
     streampos posNext (pos);
     posNext += m_nTotalSize;
@@ -315,7 +419,7 @@ Id3V240Stream::Id3V240Stream(int nIndex, NoteColl& notes, istream& in, StringWrp
             Id3V240Frame* p (new Id3V240Frame(notes, in, pos, hasUnsynch(), posNext, m_pFileName));
             bHasLatin1NonAscii = bHasLatin1NonAscii || p->m_bHasLatin1NonAscii;
             if (-1 == p->m_nMemDataSize)
-            {
+            { // it encountered zeroes, which signals the beginning of padding //ttt2 should check that there's no garbage after the first zero
                 m_nPaddingSize = posNext - pos;
                 delete p;
                 break;
@@ -379,13 +483,22 @@ Id3V240Stream::Id3V240Stream(int nIndex, NoteColl& notes, istream& in, StringWrp
     default:;
     }
 
-    /*if (0 != findFrame(KnownFrames::LBL_IMAGE()))
+    if (m_vpFrames.empty())
     {
-        MP3_NOTE (m_pos, "APIC in ID3 2.4.0 has never been tested. It may not work."); //ttt1 test & remove
-    }*/
+        MP3_NOTE (m_pos, id3v2EmptyTag);
+    }
 
     MP3_TRACE (m_pos, "Id3V240Stream built.");
 
+    {
+        pos = m_pos;
+        pos += m_nTotalSize - 1;
+        in.seekg(pos);
+        char c;
+
+        MP3_CHECK (1 == read(in, &c, 1), m_pos, id3v240CantReadFrame, NotId3V2());
+    }
+
     rst.setOk();
 }
 
@@ -405,6 +518,7 @@ Id3V240Stream::Id3V240Stream(int nIndex, NoteColl& notes, istream& in, StringWrp
     case ALBUM:
     case RATING:
     case COMPOSER:
+    case VARIOUS_ARTISTS:
         return READ_ONLY;
 
     default:
