@@ -20,6 +20,8 @@
  ***************************************************************************/
 
 
+#include  <algorithm>
+
 #include  <QScrollArea>
 #include  <QScrollBar>
 #include  <QPainter>
@@ -69,6 +71,7 @@ CurrentAlbumModel::CurrentAlbumModel(TagEditorDlgImpl* pTagEditorDlgImpl) : m_pT
 
 /*override*/ int CurrentAlbumModel::rowCount(const QModelIndex&) const
 {
+//qDebug("rowCount ret %d", cSize(m_pTagWriter->m_vpMp3HandlerTagData));
     return cSize(m_pTagWriter->m_vpMp3HandlerTagData);
 }
 
@@ -85,11 +88,13 @@ LAST_STEP("CurrentAlbumModel::data()");
     if (!index.isValid()) { return QVariant(); }
     int i (index.row());
     int j (index.column());
+//qDebug("ndx %d %d", i, j);
 
     if (Qt::DisplayRole != nRole && Qt::ToolTipRole != nRole && Qt::EditRole != nRole) { return QVariant(); }
     QString s;
 
     if (m_pTagEditorDlgImpl->isSaving()) { return "N/A"; }
+    if (m_pTagEditorDlgImpl->isNavigating()) { return ""; }
 
     if (0 == j)
     {
@@ -159,7 +164,7 @@ LAST_STEP("CurrentAlbumModel::headerData");
     catch (const Mp3HandlerTagData::InvalidValue&)
     {
         QMessageBox::critical(m_pTagEditorDlgImpl, "Error", "The data contained errors and couldn't be saved"); //ttt2 put focus on album table
-        return false; // ttt1 if it gets here the data is lost; perhaps CurrentAlbumDelegate should be modified more extensively, to not close the editor on Enter if this returns false;
+        return false; // ttt2 if it gets here the data is lost; perhaps CurrentAlbumDelegate should be modified more extensively, to not close the editor on Enter if this returns false;
     }
 }
 
@@ -196,6 +201,7 @@ LAST_STEP("CurrentFileModel::data()");
     if (Qt::DisplayRole != nRole && Qt::ToolTipRole != nRole) { return QVariant(); }
 
     if (m_pTagEditorDlgImpl->isSaving()) { return "N/A"; }
+    if (m_pTagEditorDlgImpl->isNavigating()) { return ""; }
 
     if (0 == m_pTagWriter->getCurrentHndl()) { return QVariant(); } // may happen in transient states (prev/next album)
 
@@ -286,22 +292,24 @@ LAST_STEP("CurrentFileModel::headerData");
 //======================================================================================================================
 
 
-
-
-TagEditorDlgImpl::TagEditorDlgImpl(QWidget* pParent, CommonData* pCommonData, TransfConfig& transfConfig) : QDialog(pParent, getDialogWndFlags()), Ui::TagEditorDlg(), m_pCommonData(pCommonData), m_bSectionMovedLock(false), m_transfConfig(transfConfig), m_bIsFastSaving(false), m_bIsSaving(false)
+TagEditorDlgImpl::TagEditorDlgImpl(QWidget* pParent, CommonData* pCommonData, TransfConfig& transfConfig, bool& bDataSaved) : QDialog(pParent, getDialogWndFlags()), Ui::TagEditorDlg(), m_pCommonData(pCommonData), m_bSectionMovedLock(false), m_transfConfig(transfConfig), m_bIsFastSaving(false), m_bIsSaving(false), m_bIsNavigating(false), m_bDataSaved(bDataSaved), m_bWaitingAlbumResize(false), m_bWaitingFileResize(false), m_eArtistsCase(TC_NONE), m_eOthersCase(TC_NONE)
 {
     setupUi(this);
 
+    m_bDataSaved = false;
+
+    setupVarArtistsBtn();
 
     m_pAssgnBtnWrp = new AssgnBtnWrp (m_pToggleAssignedB);
 
     {
-        m_pTagWriter = new TagWriter(m_pCommonData, this, m_bIsFastSaving);
+        m_pTagWriter = new TagWriter(m_pCommonData, this, m_bIsFastSaving, m_eArtistsCase, m_eOthersCase);
         loadTagWriterInf();
         connect(m_pTagWriter, SIGNAL(albumChanged()), this, SLOT(onAlbumChanged()));
         connect(m_pTagWriter, SIGNAL(fileChanged()), this, SLOT(onFileChanged()));
         connect(m_pTagWriter, SIGNAL(imagesChanged()), this, SLOT(onImagesChanged()));
         connect(m_pTagWriter, SIGNAL(requestSave()), this, SLOT(on_m_pSaveB_clicked()));
+        connect(m_pTagWriter, SIGNAL(varArtistsUpdated(bool)), this, SLOT(onVarArtistsUpdated(bool)));
     }
 
     m_pCurrentAlbumModel = new CurrentAlbumModel(this);
@@ -309,8 +317,8 @@ TagEditorDlgImpl::TagEditorDlgImpl(QWidget* pParent, CommonData* pCommonData, Tr
 
     {
         m_pCurrentAlbumG->verticalHeader()->setResizeMode(QHeaderView::Interactive);
-        m_pCurrentAlbumG->verticalHeader()->setMinimumSectionSize(CELL_HEIGHT + 1);
-        m_pCurrentAlbumG->verticalHeader()->setDefaultSectionSize(CELL_HEIGHT + 1);//*/
+        m_pCurrentAlbumG->verticalHeader()->setMinimumSectionSize(CELL_HEIGHT);
+        m_pCurrentAlbumG->verticalHeader()->setDefaultSectionSize(CELL_HEIGHT);//*/
 
         m_pCurrentAlbumG->setModel(m_pCurrentAlbumModel);
 
@@ -318,16 +326,17 @@ TagEditorDlgImpl::TagEditorDlgImpl(QWidget* pParent, CommonData* pCommonData, Tr
         m_pCurrentAlbumG->setItemDelegate(m_pAlbumDel);
 
 
-        //connect(m_pCurrentAlbumG, SIGNAL(clicked(const QModelIndex &)), this, SLOT(onAlbSelChanged())); // ttt1 see if both this and next are needed (next seems enough)
+        //connect(m_pCurrentAlbumG, SIGNAL(clicked(const QModelIndex &)), this, SLOT(onAlbSelChanged())); // ttt2 see if both this and next are needed (next seems enough)
         connect(m_pCurrentAlbumG->selectionModel(), SIGNAL(selectionChanged(const QItemSelection&, const QItemSelection&)), this, SLOT(onAlbSelChanged()));
         connect(m_pCurrentAlbumG->selectionModel(), SIGNAL(currentChanged(const QModelIndex&, const QModelIndex &)), this, SLOT(onAlbCrtChanged()));
     }
 
 
     {
+        m_pCurrentFileG->setVerticalHeader(new NoCropHeaderView(m_pCurrentFileG));
         m_pCurrentFileG->verticalHeader()->setResizeMode(QHeaderView::Interactive);
-        m_pCurrentFileG->verticalHeader()->setMinimumSectionSize(CELL_HEIGHT + 1);
-        m_pCurrentFileG->verticalHeader()->setDefaultSectionSize(CELL_HEIGHT + 1);//*/
+        m_pCurrentFileG->verticalHeader()->setMinimumSectionSize(CELL_HEIGHT);
+        m_pCurrentFileG->verticalHeader()->setDefaultSectionSize(CELL_HEIGHT);//*/
 
         m_pCurrentFileG->setModel(m_pCurrentFileModel);
         CurrentFileDelegate* pDel (new CurrentFileDelegate(m_pCurrentFileG, m_pCommonData));
@@ -335,14 +344,17 @@ TagEditorDlgImpl::TagEditorDlgImpl(QWidget* pParent, CommonData* pCommonData, Tr
         connect(m_pCurrentFileG->horizontalHeader(), SIGNAL(sectionMoved(int, int, int)), this, SLOT(onFileSelSectionMoved(int, int, int)));
 
         m_pCurrentFileG->setItemDelegate(pDel);
+
+        m_pCurrentFileG->horizontalHeader()->setMovable(true);
     }
 
-    m_pCurrentFileG->horizontalHeader()->setMovable(true);
 
     {
         int nWidth, nHeight;
-        QByteArray vPrevState;
-        m_pCommonData->m_settings.loadTagEdtSettings(nWidth, nHeight, vPrevState);
+        int nArtistsCase, nOthersCase;
+        m_pCommonData->m_settings.loadTagEdtSettings(nWidth, nHeight, nArtistsCase, nOthersCase);
+        m_eArtistsCase = TextCaseOptions(nArtistsCase);
+        m_eOthersCase = TextCaseOptions(nOthersCase);
         if (nWidth > 400 && nHeight > 400)
         {
             resize(nWidth, nHeight);
@@ -351,17 +363,11 @@ TagEditorDlgImpl::TagEditorDlgImpl(QWidget* pParent, CommonData* pCommonData, Tr
         {
             defaultResize(*this);
         }
-
-        if (!vPrevState.isNull())
-        {
-            m_pTagEdtSplitter->restoreState(vPrevState);
-        }
-        m_pTagEdtSplitter->setOpaqueResize(false);
     }
 
 
     {
-        delete m_pRemovableL; // m_pRemovableL's only job is to make m_pImagesW visible in QtDesigner
+        delete m_pRemovable1L; // m_pRemovableL's only job is to make m_pImagesW visible in QtDesigner
 
         QWidget* pWidget (new QWidget (this));
         pWidget->setSizePolicy(QSizePolicy(QSizePolicy::MinimumExpanding, QSizePolicy::MinimumExpanding));
@@ -369,7 +375,7 @@ TagEditorDlgImpl::TagEditorDlgImpl(QWidget* pParent, CommonData* pCommonData, Tr
         QHBoxLayout* pScrlLayout (new QHBoxLayout (pWidget));
         pWidget->setLayout(pScrlLayout);
 
-        pWidget->layout()->setContentsMargins(0, 0, 0, 4 + 1); // "+1" added to look better, but don't know how to calculated
+        pWidget->layout()->setContentsMargins(0, 0, 0, 4 + 1); // "+1" added to look better, but don't know how to calculate
 
         m_pImgScrollArea = new QScrollArea (m_pImagesW);
 
@@ -379,6 +385,11 @@ TagEditorDlgImpl::TagEditorDlgImpl(QWidget* pParent, CommonData* pCommonData, Tr
         m_pImgScrollArea->setFrameShape(QFrame::NoFrame);
     }
 
+    {
+        delete m_pRemovable2L; // m_pRemovable2L's only job is to make m_pPatternsW visible in QtDesigner
+        createPatternButtons();
+    }
+
     //layout()->update();
     /*layout()->update();
     widget_4->layout()->update();
@@ -395,18 +406,79 @@ TagEditorDlgImpl::TagEditorDlgImpl(QWidget* pParent, CommonData* pCommonData, Tr
     m_pCurrentAlbumG->installEventFilter(this);
     installEventFilter(this);
 
+    {
+        int nFileHght ((CELL_HEIGHT)*10 + m_pCurrentFileG->horizontalHeader()->height() + 2*QApplication::style()->pixelMetric(QStyle::PM_DefaultFrameWidth, 0, m_pCurrentFileG));
+        m_pCurrentFileG->setMaximumHeight(nFileHght);
+        m_pCurrentFileG->setMinimumHeight(nFileHght);
+    }
+
     { QAction* p (new QAction(this)); p->setShortcut(QKeySequence("F1")); connect(p, SIGNAL(triggered()), this, SLOT(onHelp())); addAction(p); }
 
     QTimer::singleShot(1, this, SLOT(onShow())); // just calls resizeTagEditor(); !!! needed to properly resize the table columns; album and file tables have very small widths until they are actually shown, so calling resizeTagEditor() earlier is pointless; calling update() on various layouts seems pointless as well; (see also DoubleList::resizeEvent() )
 }
 
 
+void TagEditorDlgImpl::createPatternButtons()
+{
+    QBoxLayout* pLayout (dynamic_cast<QBoxLayout*>(m_pPatternsW->layout()));
+    CB_ASSERT (0 != pLayout);
+
+    QObjectList l (m_pPatternsW->children());
+
+    for (int i = 0, n = l.size(); i < n; ++i)
+    {
+        if (l[i] != pLayout)
+        {
+            delete l[i];
+        }
+    }
+
+    const set<int>& snActivePatterns (m_pTagWriter->getActivePatterns());
+
+    m_vpPattButtons.clear();
+
+    const vector<string>& vstrPatterns (m_pTagWriter->getPatterns());
+    int n (cSize(vstrPatterns));
+    for (int i = 0; i < n; ++i)
+    {
+        QToolButton* p (new QToolButton(m_pPatternsW));
+        p->setText(toNativeSeparators(convStr(vstrPatterns[i])));
+        p->setCheckable(true);
+        m_vpPattButtons.push_back(p);
+        if (snActivePatterns.count(i) > 0)
+        {
+            p->setChecked(true);
+        }
+        connect(p, SIGNAL(clicked()), this, SLOT(onPatternClicked()));
+        pLayout->insertWidget(i, p);
+    }
+
+    pLayout->insertStretch(n);
+
+    m_pPatternsW->setVisible(!m_vpPattButtons.empty());
+}
+
+
+void TagEditorDlgImpl::onPatternClicked()
+{
+    set<int> s;
+    for (int i = 0; i < cSize(m_vpPattButtons); ++i)
+    {
+        if (m_vpPattButtons[i]->isChecked())
+        {
+            s.insert(i);
+        }
+    }
+
+    m_pTagWriter->setActivePatterns(s);
+}
+
 
 TagEditorDlgImpl::~TagEditorDlgImpl()
 {
     saveTagWriterInf();
-    QByteArray vState (m_pTagEdtSplitter->saveState());
-    m_pCommonData->m_settings.saveTagEdtSettings(width(), height(), vState);
+
+    m_pCommonData->m_settings.saveTagEdtSettings(width(), height(), int(m_eArtistsCase), int(m_eOthersCase));
     delete m_pCurrentAlbumModel;
     delete m_pCurrentFileModel;
     delete m_pTagWriter;
@@ -422,9 +494,32 @@ string TagEditorDlgImpl::run()
 }
 
 
+void TagEditorDlgImpl::setupVarArtistsBtn()
+{
+    m_pVarArtistsB->setEnabled(m_pCommonData->m_bItunesVarArtists || m_pCommonData->m_bWmpVarArtists);
+    m_pVarArtistsB->setToolTip(m_pVarArtistsB->isEnabled() ?
+            "Toggle \"Various Artists\"" :
+
+            "To enable \"Various Artists\" you need to open the\n"
+            "configuration dialog, go to the \"Others\" tab and\n"
+            "check the corresponding checkbox(es)");
+}
+
+
 
 void TagEditorDlgImpl::resizeTagEditor()
 {
+    if (!m_bWaitingAlbumResize)
+    {
+        m_bWaitingAlbumResize = true;
+        QTimer::singleShot(1, this, SLOT(onResizeTagEditorDelayed()));
+    }
+}
+
+
+void TagEditorDlgImpl::onResizeTagEditorDelayed() // needed because it's pointless to call this while navigating, when the text for all cells is returned as ""
+{
+    m_bWaitingAlbumResize = false;
     QWidget* pParent (m_pCurrentAlbumG->parentWidget());
 
     //cout << "\n========================\n" << endl; listWidget(pParent);
@@ -437,12 +532,12 @@ void TagEditorDlgImpl::resizeTagEditor()
         int nScrollBarWidth (QApplication::style()->pixelMetric(QStyle::PM_ScrollBarExtent));
         /*if (0 != QApplication::style()->styleHint(QStyle::SH_ScrollView_FrameOnlyAroundContents)) // see if this is needed for Oxygen
         {
-            nRes -= 2*QApplication::style()->pixelMetric(QStyle::PM_DefaultFrameWidth, 0, &m_tbl); //ttt1 Qt 4.4 (and below) - specific; in 4.5 there's a QStyle::PM_ScrollView_ScrollBarSpacing
+            nRes -= 2*QApplication::style()->pixelMetric(QStyle::PM_DefaultFrameWidth, 0, &m_tbl); //ttt2 Qt 4.4 (and below) - specific; in 4.5 there's a QStyle::PM_ScrollView_ScrollBarSpacing
         }*/
 
         nHeight += nScrollBarWidth;
     }
-    m_pImagesW->setMaximumSize(100000, nHeight); // ttt1 see about adding space for horizontal scrollbar
+    m_pImagesW->setMaximumSize(100000, nHeight); // ttt2 see about adding space for horizontal scrollbar
 
 
     pParent->layout()->activate(); // resize m_pCurrentAlbumG based on the image list being visible or not
@@ -468,6 +563,16 @@ intf1.setFixedWidth(5, 50);*/
 
 void TagEditorDlgImpl::resizeFile() // resizes the "current file" grid; called by resizeTagEditor() and by onFileChanged();
 {
+    if (!m_bWaitingFileResize)
+    {
+        m_bWaitingFileResize = true;
+        QTimer::singleShot(1, this, SLOT(onResizeFileDelayed()));
+    }
+}
+
+void TagEditorDlgImpl::onResizeFileDelayed() // resizes the "current file" grid; called by resizeTagEditor() and by onFileChanged();
+{
+    m_bWaitingFileResize = false;
     SimpleQTableViewWidthInterface intf2 (*m_pCurrentFileG);
     ColumnResizer rsz2 (intf2, 100, ColumnResizer::DONT_FILL, ColumnResizer::CONSISTENT_RESULTS);
 }
@@ -620,6 +725,7 @@ void TagEditorDlgImpl::on_m_pEditPatternsB_clicked()
     if (dlg.run(v))
     {
         m_pTagWriter->updatePatterns(v);
+        createPatternButtons();
     }
 }
 
@@ -659,6 +765,16 @@ void TagEditorDlgImpl::onImagesChanged()
 }
 
 
+
+void TagEditorDlgImpl::onVarArtistsUpdated(bool bVarArtists)
+{
+    static QPixmap picSa (":/images/va_sa.svg");
+    static QPixmap picVa (":/images/va_va.svg");
+
+    m_pVarArtistsB->setIcon(bVarArtists ? picVa : picSa);
+}
+
+
 void TagEditorDlgImpl::onAlbumChanged(/*bool bContentOnly*/)
 {
     if (m_pTagWriter->m_vpMp3HandlerTagData.empty())
@@ -671,13 +787,24 @@ void TagEditorDlgImpl::onAlbumChanged(/*bool bContentOnly*/)
     m_pCurrentAlbumModel->emitLayoutChanged();
 
     const Mp3Handler* p (m_pTagWriter->m_vpMp3HandlerTagData.at(0)->getMp3Handler());
-    m_pCrtDirTagEdtE->setText(toNativeSeparators(convStr(p->getDir())));
+    QString qs (toNativeSeparators(convStr(p->getDir())));
+#ifndef WIN32
+#else
+    if (2 == qs.size() && ':' == qs[1])
+    {
+        qs += "\\";
+    }
+#endif
+    m_pCrtDirTagEdtE->setText(qs);
 }
 
+
 void TagEditorDlgImpl::on_m_pConfigB_clicked()
 {
     if (!closeEditor()) { return; }
 
+    SaveOpt eSaveOpt (save(IMPLICIT)); if (SAVED != eSaveOpt && DISCARDED != eSaveOpt) { return; }
+
     ConfigDlgImpl dlg (m_transfConfig, m_pCommonData, this, ConfigDlgImpl::SOME_TABS);
 
     if (dlg.run())
@@ -687,6 +814,9 @@ void TagEditorDlgImpl::on_m_pConfigB_clicked()
         resizeIcons();
         resizeTagEditor();
     }
+
+    m_pTagWriter->reloadAll("", TagWriter::CLEAR_DATA, TagWriter::CLEAR_ASSGN); // !!! regardless of the conf being cancelled, so it's consistent with the question asked at the beginning
+    setupVarArtistsBtn();
 }
 
 
@@ -793,8 +923,18 @@ void TagEditorDlgImpl::loadTagWriterInf()
         }*/
 
         bErr = !m_pTagWriter->updatePatterns(v) || bErr;
+
+        bool bErr2;
+        vector<string> vstrActivePatterns (m_pCommonData->m_settings.loadVector("tagWriter/activePatterns", bErr2));
+        set<int> snActivePatterns;
+        for (int i = 0; i < cSize(vstrActivePatterns); ++i)
+        {
+            snActivePatterns.insert(atoi(vstrActivePatterns[i].c_str()));
+        }
+        m_pTagWriter->setActivePatterns(snActivePatterns);
+
 //if (bErr) { qDebug("error when reading patterns"); } //ttt remove
-        if (bErr)
+        if (bErr || bErr2)
         {
             QMessageBox::warning(this, "Error setting up patterns", "An invalid value was found in the configuration file. You'll have to set up the patterns manually.");
         }
@@ -829,6 +969,16 @@ void TagEditorDlgImpl::saveTagWriterInf()
             v.push_back("*");
         }*/
         m_pCommonData->m_settings.saveVector("tagWriter/patterns", v);
+
+        vector<string> u;
+        char a [15];
+        const set<int>& snActivePatterns (m_pTagWriter->getActivePatterns());
+        for (set<int>::const_iterator it = snActivePatterns.begin(); it != snActivePatterns.end(); ++it)
+        {
+            sprintf(a, "%d", *it);
+            u.push_back(a);
+        }
+        m_pCommonData->m_settings.saveVector("tagWriter/activePatterns", u);
     }
 }
 
@@ -865,7 +1015,7 @@ void TagEditorDlgImpl::on_m_pReloadB_clicked()
 }
 
 
-//ttt0 perhaps transf to set recording time based on file date; another to set file date based on rec time
+//ttt2 perhaps transf to set recording time based on file date; another to set file date based on rec time
 
 // copies the values from the first row to the other rows for columns that have at least a cell selected (doesn't matter if more than 1 cells are selected);
 void TagEditorDlgImpl::on_m_pCopyFirstB_clicked()
@@ -876,7 +1026,59 @@ void TagEditorDlgImpl::on_m_pCopyFirstB_clicked()
 }
 
 
-void TagEditorDlgImpl::on_m_pSaveB_clicked() //ttt1 perhaps make this save selected list, by using SHIFT
+void TagEditorDlgImpl::on_m_pVarArtistsB_clicked()
+{
+    if (!closeEditor()) { return; }
+
+    m_pTagWriter->toggleVarArtists();
+}
+
+
+void TagEditorDlgImpl::on_m_pCaseB_clicked()
+{
+    QMenu menu;
+    vector<QAction*> vpAct;
+
+    QAction* pAct;
+    for (int i = TC_NONE; i <= TC_SENTENCE; ++i)
+    {
+        pAct = new QAction(QString("Artists - ") + getCaseAsStr(TextCaseOptions(i)), &menu); menu.addAction(pAct); vpAct.push_back(pAct);
+    }
+    menu.addSeparator();
+    for (int i = TC_NONE; i <= TC_SENTENCE; ++i)
+    {
+        pAct = new QAction(QString("Others - ") + getCaseAsStr(TextCaseOptions(i)), &menu); menu.addAction(pAct); vpAct.push_back(pAct);
+    }
+
+    for (int i = 0; i < cSize(vpAct); ++i) { vpAct[i]->setCheckable(true); }
+
+    CB_ASSERT(m_eArtistsCase >= TC_NONE && m_eArtistsCase <= TC_SENTENCE);
+    CB_ASSERT(m_eOthersCase >= TC_NONE && m_eOthersCase <= TC_SENTENCE);
+
+    vpAct[m_eArtistsCase - TC_NONE]->setChecked(true);
+    vpAct[m_eOthersCase - TC_NONE + 5]->setChecked(true);
+
+
+    QAction* p (menu.exec(m_pCaseB->mapToGlobal(QPoint(0, m_pCaseB->height()))));
+    if (0 != p)
+    {
+        int nIndex (std::find(vpAct.begin(), vpAct.end(), p) - vpAct.begin());
+        if (nIndex < 5)
+        {
+            m_eArtistsCase = TextCaseOptions(nIndex - 1);
+        }
+        else
+        {
+            m_eOthersCase = TextCaseOptions(nIndex - 1 - 5);
+        }
+
+        m_pTagWriter->reloadAll(m_pTagWriter->getCurrentName(), TagWriter::DONT_CLEAR_DATA, TagWriter::DONT_CLEAR_ASSGN);
+    }
+}
+
+
+
+void TagEditorDlgImpl::on_m_pSaveB_clicked() //ttt2 perhaps make this save selected list, by using SHIFT
 {
     if (!closeEditor()) { return; }
 
@@ -951,7 +1153,7 @@ void TagEditorDlgImpl::onFileSelSectionMoved(int /*nLogicalIndex*/, int nOldVisu
     pSelModel->setCurrentIndex(selNdx, QItemSelectionModel::SelectCurrent);
     m_pCurrentAlbumG->horizontalScrollBar()->setValue(nHrzPos);
     m_pCurrentAlbumG->verticalScrollBar()->setValue(nVertPos);
-//ttt1 perhaps detect when files are removed/changed; while most tags in Id3V2 are stored with the handler, the longer ones (e.g. pictures, but others might be affected too) are dumped from memory by Id3V230Frame's constructor and retrieved from disk when needed; the user should be notified when something becomes unavailable; not sure where to do this, though: Id3V2StreamBase::getImage() is good for images, but other frames might need something similar;
+//ttt2 perhaps detect when files are removed/changed; while most tags in Id3V2 are stored with the handler, the longer ones (e.g. pictures, but others might be affected too) are dumped from memory by Id3V230Frame's constructor and retrieved from disk when needed; the user should be notified when something becomes unavailable; not sure where to do this, though: Id3V2StreamBase::getImage() is good for images, but other frames might need something similar;
 }
 
 
@@ -964,9 +1166,14 @@ void TagEditorDlgImpl::on_m_pNextB_clicked()
 
     if (m_pCommonData->nextAlbum())
     {
+        ValueRestorer<bool> rst1 (m_bIsNavigating);
+        m_bIsNavigating = true;
+
         m_pTagWriter->clearShowedNonSeqWarn();
         m_pTagWriter->reloadAll("", TagWriter::CLEAR_DATA, TagWriter::CLEAR_ASSGN);
         clearSelection(); // actually here it selects the first cell
+
+        QTimer::singleShot(1, this, SLOT(onShowPatternNote()));
     }
 }
 
@@ -979,9 +1186,14 @@ void TagEditorDlgImpl::on_m_pPrevB_clicked()
 
     if (m_pCommonData->prevAlbum())
     {
+        ValueRestorer<bool> rst1 (m_bIsNavigating);
+        m_bIsNavigating = true;
+
         m_pTagWriter->clearShowedNonSeqWarn();
         m_pTagWriter->reloadAll("", TagWriter::CLEAR_DATA, TagWriter::CLEAR_ASSGN);
         clearSelection(); // actually here it selects the first cell
+
+        QTimer::singleShot(1, this, SLOT(onShowPatternNote()));
     }
 }
 
@@ -992,7 +1204,7 @@ void TagEditorDlgImpl::clearSelection()
     QItemSelectionModel* pSelModel (m_pCurrentAlbumG->selectionModel());
     pSelModel->setCurrentIndex(m_pCurrentAlbumModel->index(0, 0), QItemSelectionModel::SelectCurrent); // !!! needed because otherwise pSelModel->clear() might trigger onAlbSelChanged() for an invalid cell (e.g. when current album has 15 songs, with 12th selected, and the next only has 10)
     pSelModel->clear();
-    pSelModel->setCurrentIndex(m_pCurrentAlbumModel->index(0, 0), QItemSelectionModel::SelectCurrent); // !!! needed to have a cell selected //ttt1 try and get rid of  duplicate call to setCurrentIndex;
+    pSelModel->setCurrentIndex(m_pCurrentAlbumModel->index(0, 0), QItemSelectionModel::SelectCurrent); // !!! needed to have a cell selected //ttt2 try and get rid of  duplicate call to setCurrentIndex;
 }
 
 
@@ -1053,6 +1265,8 @@ void TagEditorDlgImpl::resizeIcons()
     v.push_back(m_pConfigB);
     v.push_back(m_pSaveB);
     v.push_back(m_pReloadB);
+    v.push_back(m_pVarArtistsB);
+    v.push_back(m_pCaseB);
     v.push_back(m_pCopyFirstB);
     v.push_back(m_pSortB);
     v.push_back(m_pToggleAssignedB);
@@ -1099,7 +1313,7 @@ void TagEditorDlgImpl::resizeIcons()
 //static int s_nCnt (0);
     /*if (0 != pKeyEvent && Qt::Key_Escape == nKey)
     {
-        qDebug("%d. %s %d", s_nCnt++, pObj->objectName().toUtf8().data(), (int)pEvent->type()); //return QDialog::eventFilter(pObj, pEvent);
+        qDebug("%d. %s %d", s_nCnt++, pObj->objectName().toUtf8().constData(), (int)pEvent->type()); //return QDialog::eventFilter(pObj, pEvent);
     }*/
 static bool bIgnoreNextEsc (false);
     if (0 != pKeyEvent && Qt::Key_Escape == nKey && this == pObj && QEvent::ShortcutOverride == pEvent->type())
@@ -1111,7 +1325,7 @@ static bool bIgnoreNextEsc (false);
         //qDebug("kill esc");
         //return true;
         qDebug(" >> save");
-        //qDebug("%d. %s %d", s_nCnt++, pObj->objectName().toUtf8().data(), (int)pEvent->type()); //return QDialog::eventFilter(pObj, pEvent);
+        //qDebug("%d. %s %d", s_nCnt++, pObj->objectName().toUtf8().constData(), (int)pEvent->type()); //return QDialog::eventFilter(pObj, pEvent);
         //if (this == pObj)
         {
             SaveOpt eSaveOpt (save(false));
@@ -1132,7 +1346,7 @@ static bool bIgnoreNextEsc (false);
 e1:
     if (0 != pKeyEvent && Qt::Key_Escape == nKey)
     {
-        //qDebug("passed through: %d. %s %d", s_nCnt++, pObj->objectName().toUtf8().data(), (int)pEvent->type());
+        //qDebug("passed through: %d. %s %d", s_nCnt++, pObj->objectName().toUtf8().constData(), (int)pEvent->type());
     }
 
     if (0 != pKeyEvent && Qt::Key_Escape == nKey && this == pObj && QEvent::KeyPress == pEvent->type())
@@ -1160,7 +1374,7 @@ e1:
         //&& Qt::Key_Escape == nKey
         )
     {
-        qDebug("%d. %s %d", s_nCnt++, pObj->objectName().toUtf8().data(), (int)pEvent->type()); //return QDialog::eventFilter(pObj, pEvent);
+        qDebug("%d. %s %d", s_nCnt++, pObj->objectName().toUtf8().constData(), (int)pEvent->type()); //return QDialog::eventFilter(pObj, pEvent);
     }//*/
 
     if (0 != pKeyEvent && Qt::Key_Escape == nKey && this == pObj && QEvent::KeyPress == pEvent->type())
@@ -1195,7 +1409,7 @@ void TagEditorDlgImpl::eraseSelFields() // erases the values in the selected fie
     m_pAssgnBtnWrp->setState(m_pTagWriter->updateAssigned(vector<pair<int, int> >())); // we don't want to keep any previous value
     updateAssigned(); // needed for the "assign" button to work, because the previous line cleared m_pTagWriter->m_sSelOrigVal
 
-    m_pTagWriter->reloadAll(m_pTagWriter->getCurrentName(), TagWriter::DONT_CLEAR_DATA, TagWriter::DONT_CLEAR_ASSGN); //ttt1 way too many ugly calls, including 2 required calls to m_pAssgnBtnWrp->setState(); restructure the whole "assigned" thing;
+    m_pTagWriter->reloadAll(m_pTagWriter->getCurrentName(), TagWriter::DONT_CLEAR_DATA, TagWriter::DONT_CLEAR_ASSGN); //ttt2 way too many ugly calls, including 2 required calls to m_pAssgnBtnWrp->setState(); restructure the whole "assigned" thing;
     /*
         some details (not completely up-to-date):
             TagWriter::toggleAssigned() should be called when the user clicks on the assign button; changes status of selected cells and returns the new state of m_pToggleAssignedB
@@ -1217,7 +1431,7 @@ void TagEditorDlgImpl::eraseSelFields() // erases the values in the selected fie
 
 
 
-//ttt1 disabled widgets should have tooltips saying why are they disabled / how to enable them
+//ttt2 disabled widgets should have tooltips saying why are they disabled / how to enable them
 //===================================================================================================================
 
 
@@ -1235,6 +1449,7 @@ public:
     /*override*/ Transformation::Result apply(const Mp3Handler&, const TransfConfig&, const std::string& strOrigSrcName, std::string& strTempName);
     /*override*/ const char* getActionName() const { return getClassName(); }
     /*override*/ const char* getDescription() const { return "Saves user-edited ID3V2.3.0 tags."; }
+    /*override*/ bool acceptsFastSave() const { return true; }
 
     static const char* getClassName() { return "Save ID3V2.3.0 tags"; }
 };
@@ -1249,12 +1464,13 @@ void Id3V230Writer::setupWriter(Id3V230StreamWriter& wrt, const Mp3HandlerTagDat
     s = pMp3HandlerTagData->getData(TagReader::GENRE); if (s.empty()) { wrt.removeFrames(KnownFrames::LBL_GENRE()); } else { wrt.addTextFrame(KnownFrames::LBL_GENRE(), s); }
     s = pMp3HandlerTagData->getData(TagReader::ALBUM); if (s.empty()) { wrt.removeFrames(KnownFrames::LBL_ALBUM()); } else { wrt.addTextFrame(KnownFrames::LBL_ALBUM(), s); }
     s = pMp3HandlerTagData->getData(TagReader::COMPOSER); if (s.empty()) { wrt.removeFrames(KnownFrames::LBL_COMPOSER()); } else { wrt.addTextFrame(KnownFrames::LBL_COMPOSER(), s); }
+    s = pMp3HandlerTagData->getData(TagReader::VARIOUS_ARTISTS); wrt.setVariousArtists(!s.empty());
     s = pMp3HandlerTagData->getData(TagReader::TIME); if (s.empty()) { wrt.removeFrames(KnownFrames::LBL_TIME_DATE_230()); wrt.removeFrames(KnownFrames::LBL_TIME_YEAR_230()); wrt.removeFrames(KnownFrames::LBL_TIME_240()); } else { wrt.setRecTime(TagTimestamp(s)); }
 
     s = pMp3HandlerTagData->getData(TagReader::IMAGE);
     if (s.empty())
     {
-        wrt.removeFrames(KnownFrames::LBL_IMAGE(), Id3V2Frame::COVER);
+        wrt.removeFrames(KnownFrames::LBL_IMAGE(), -1); // !!! this removes all APIC frames, including those non-cover frames that got assigned to Id3V2StreamBase::m_pPicFrame
     }
     else
     {
@@ -1274,10 +1490,10 @@ void Id3V230Writer::setupWriter(Id3V230StreamWriter& wrt, const Mp3HandlerTagDat
         case ImageInfo::PNG: szEncoding = "image/png"; break;
         case ImageInfo::INVALID:
             {
-                QPixmap scaledPic;
-                ImageInfo::compress(imgInfo.getPixmap(), scaledPic, recomprImg);
+                QImage scaledPic;
+                ImageInfo::compress(imgInfo.getImage(), scaledPic, recomprImg);
                 nImgSize = recomprImg.size();
-                pImgData = recomprImg.data();
+                pImgData = recomprImg.constData();
                 szEncoding = "image/jpg";
                 break;
             }
@@ -1291,10 +1507,10 @@ void Id3V230Writer::setupWriter(Id3V230StreamWriter& wrt, const Mp3HandlerTagDat
         *q++ = 0; // enc
         strcpy(q, szEncoding);
         q += nEncSize + 1; // enc + term
-        *q++ = Id3V2Frame::COVER;
+        *q++ = Id3V2Frame::PT_COVER;
         *q++ = 0; // null-term descr
         memcpy(q, pImgData, nImgSize);
-        wrt.addImage(frm);
+        wrt.addImg(frm);
     }
 
     {
@@ -1316,9 +1532,11 @@ void Id3V230Writer::setupWriter(Id3V230StreamWriter& wrt, const Mp3HandlerTagDat
         }
     }
 }
+//ttt2 when multiple id3v2 are found, the bkg color doesn't change if the 2 id3v2 are switched; probably ok, though, in the sense that if at least one field is assigned, all the others will be used when saving
 
 /*override*/ Transformation::Result Id3V230Writer::apply(const Mp3Handler& h, const TransfConfig& transfConfig, const std::string& strOrigSrcName, std::string& strTempName)
 {
+    LAST_STEP("Id3V230Writer::apply() " + h.getName());
     const vector<DataStream*>& vpStreams (h.getStreams());
 
     const Mp3HandlerTagData* pMp3HandlerTagData (0);
@@ -1348,7 +1566,7 @@ e1:
             if (0 != pId3V2Source) { break; }
         }
 
-        Id3V230StreamWriter wrt (m_bKeepOneValidImg, m_bFastSave, pId3V2Source); // OK if pId3V2Source is 0
+        Id3V230StreamWriter wrt (m_bKeepOneValidImg, m_bFastSave, pId3V2Source, h.getName()); // OK if pId3V2Source is 0
 
         setupWriter(wrt, pMp3HandlerTagData);
 
@@ -1367,10 +1585,15 @@ e1:
 
                 h.reloadId3V2(); //ttt2 perhaps return the pointer to the old Id3V2 instead of destroying it, and keep it somewhere until saving is done; then m_bIsFastSaving won't be needed, and no "N/A"s will be displayed in ID3V2 fields when saving; OTOH those pointers can't be kept fully alive, because they can't retrieve their data from disk; so better leave it as is;
 
-                if (transfConfig.m_optionsWrp.m_opt.m_bKeepOrigTime)
+                if (transfConfig.m_options.m_bKeepOrigTime)
                 {
                     setFileDate(h.getName(), nOrigTime);
                 }
+                else
+                {
+                    getFileInfo(h.getName(), nOrigTime, nSize);
+                    h.m_nFastSaveTime = nOrigTime;
+                }
 
                 return NOT_CHANGED; // !!!
             }
@@ -1413,7 +1636,7 @@ e1:
 //========================================================================================================================
 //========================================================================================================================
 
-//ttt1 perhaps all licences in root dir
+//ttt2 perhaps all licences in root dir
 
 // based on configuration, either just saves the tags or asks the user for confirmation; returns true iff all tags have been saved or if none needed saving; it should be followed by a reload(), either for the current or for the next/prev album; if bExplicitCall is true, the "ASK" option is turned into "SAVE";
 TagEditorDlgImpl::SaveOpt TagEditorDlgImpl::save(bool bImplicitCall)
@@ -1440,7 +1663,7 @@ TagEditorDlgImpl::SaveOpt TagEditorDlgImpl::save(bool bImplicitCall)
         bHasUnsavedAssgn = bHasUnsavedAssgn || bAssgn;
         bHasUnsavedNonId3V2 = bHasUnsavedNonId3V2 || bNonId3V2;
     }
-//ttt1 perhaps separate setting for showing and saving non-id3v2 fields
+//ttt2 perhaps separate setting for showing and saving non-id3v2 fields
 
     {
         int nUnassignedImagesCnt (m_pTagWriter->getUnassignedImagesCount());
@@ -1482,6 +1705,8 @@ TagEditorDlgImpl::SaveOpt TagEditorDlgImpl::save(bool bImplicitCall)
         if (2 == nOpt) { return CANCELLED; }
     }
 
+    m_bDataSaved = true;
+
     Id3V230Writer wrt (m_pTagWriter, m_pCommonData->m_bKeepOneValidImg, m_pCommonData->useFastSave());
     vector<Transformation*> vpTransf;
     vpTransf.push_back(&wrt);
@@ -1505,13 +1730,29 @@ TagEditorDlgImpl::SaveOpt TagEditorDlgImpl::save(bool bImplicitCall)
     return bRes ? SAVED : PARTIALLY_SAVED;
 }
 
-
+//ttt2 hide VA column if not used
 void TagEditorDlgImpl::onHelp()
 {
     openHelp("190_tag_editor.html");
 }
 
 
+extern bool s_bToldAboutPatternsInCrtRun;
+
+void TagEditorDlgImpl::onShowPatternNote()
+{
+    if (m_pTagWriter->shouldShowPatternsNote())
+    {
+        s_bToldAboutPatternsInCrtRun = true;
+
+        HtmlMsg::msg(this, 0, 0, &m_pCommonData->m_bToldAboutPatterns, HtmlMsg::DEFAULT, "Info", "<p style=\"margin-bottom:1px; margin-top:12px; \">Some fields are missing or may be incomplete. While this is usually solved by downloading correct information, there are a cases when this approach doesn't work, like custom compilations, rare albums, or missing tracks.</p>"
+
+        "<p style=\"margin-bottom:1px; margin-top:12px; \">If your current folder fits one of these cases or you simply have consistently named files that you would prefer to use as a source of track info, you may want to take a look at the tag editor's patterns, at <a href=\"http://mp3diags.sourceforge.net/220_tag_editor_patterns.html\">http://mp3diags.sourceforge.net/220_tag_editor_patterns.html</a>.</p>", 550, 300, "OK"); //ttt1 different for "unstable"
+    }
+}
+
+
+
 
 //========================================================================================================================================================
 //========================================================================================================================================================
@@ -1551,6 +1792,22 @@ CurrentFileDelegate::CurrentFileDelegate(QTableView* pTableView, const CommonDat
         QItemDelegate::paint(pPainter, option, index);
     }
 
+
+    /*
+    //ttt2 first 3 cases don't show the dotted line because QItemDelegate::paint() doesn't get called; perhaps drawDecoration() should be called, but copying these things from qitemdelegate.cpp didn't help:
+
+    QVariant value;
+    value = index.data(Qt::DecorationRole);
+    QPixmap pixmap;
+    pixmap = decoration(option, value);
+
+    drawDecoration(pPainter, option, QRect(0, 0, 15, 15), pixmap);
+
+??? or perhaps use     QApplication::style()->drawPrimitive(QStyle::PE_FrameFocusRect?, &option, pPainter);
+
+
+    */
+
     pPainter->restore();
 }
 
@@ -1566,7 +1823,7 @@ return QItemDelegate::sizeHint(option, index);
 
     int j (index.column());
     int nColWidth (m_pTableView->horizontalHeader()->sectionSize(j));
-    //QRect r (0, 0, nColWidth - 2*nMargin - 1, 10000); // !!! this "-1" is what's different from Qt's implementation //ttt1 see if this is fixed in 4.4 2008.30.06 - apparently it's not fixed and the workaround no longer works
+    //QRect r (0, 0, nColWidth - 2*nMargin - 1, 10000); // !!! this "-1" is what's different from Qt's implementation //ttt2 see if this is fixed in 4.4 2008.30.06 - apparently it's not fixed and the workaround no longer works
 
     //QSize res (option.fontMetrics.boundingRect(r, Qt::AlignTop | Qt::TextWordWrap, index.data(Qt::DisplayRole).toString()).size());
 
@@ -1595,7 +1852,7 @@ CurrentAlbumDelegate::CurrentAlbumDelegate(QTableView* pTableView, TagEditorDlgI
 
 /*override*/ void CurrentAlbumDelegate::paint(QPainter* pPainter, const QStyleOptionViewItem& option, const QModelIndex& index) const
 {
-    if (m_pTagEditorDlgImpl->isSaving())
+    if (m_pTagEditorDlgImpl->isSaving() || m_pTagEditorDlgImpl->isNavigating())
     {
         QItemDelegate::paint(pPainter, option, index);
         return;
@@ -1633,12 +1890,12 @@ CurrentAlbumDelegate::CurrentAlbumDelegate(QTableView* pTableView, TagEditorDlgI
 
     pPainter->restore();
 }
-//ttt1 2009.04.06 - for all delegates: see if they can be removed by using the standard delegate and adding more functionality to the models; see Qt::ForegroundRole, Qt::TextAlignmentRole, Qt::FontRole & Co
+//ttt2 2009.04.06 - for all delegates: see if they can be removed by using the standard delegate and adding more functionality to the models; see Qt::ForegroundRole, Qt::TextAlignmentRole, Qt::FontRole & Co
 
 /*override*/ QWidget* CurrentAlbumDelegate::createEditor(QWidget* pParent, const QStyleOptionViewItem& style, const QModelIndex& index) const
 {
     int nField (index.column());
-    if (0 == nField || 7 == nField) { return 0; }
+    if (0 == nField || 5 == nField || 8 == nField) { return 0; }
     QWidget* pEditor (QItemDelegate::createEditor(pParent, style, index));
     //qDebug("%s", p->metaObject()->className());
     connect(pEditor, SIGNAL(destroyed(QObject*)), this, SLOT(onEditorDestroyed(QObject*)));
@@ -1663,15 +1920,15 @@ bool CurrentAlbumDelegate::closeEditor() // closes the editor opened with F2, sa
     }
     //delete m_pEditor;
 
-    if (0 != showMessage(m_pTableView, QMessageBox::Warning, 1, 1, "Warning", "You are editing data in a cell. If you proceed that change will be lost. Proceed and lose the data?", "Proceed", "Cancel")) { return false; } //ttt1 try and post somehow the content of the editor (?? perhaps send it a keyboard message that ENTER was pressed)
-    //ttt1 review this
+    if (0 != showMessage(m_pTableView, QMessageBox::Warning, 1, 1, "Warning", "You are editing data in a cell. If you proceed that change will be lost. Proceed and lose the data?", "Proceed", "Cancel")) { return false; } //ttt2 try and post somehow the content of the editor (?? perhaps send it a keyboard message that ENTER was pressed)
+    //ttt2 review this
 
     delete *m_spEditors.begin();
     return true;
 }
 
 
-//ttt1 perhaps manufacture track numbers when pasting tables, if track numbers don't exist
+//ttt2 perhaps manufacture track numbers when pasting tables, if track numbers don't exist
 
 
 
@@ -1703,4 +1960,4 @@ A less important performance issue is in ImageInfoPanelWdgImpl::ImageInfoPanelWd
 
 */
 
-//ttt0 perhaps check boxes or something to have many patterns defined, yet several used at a time; one idea: another window where fields are shown for all the patterns and the user can pick; other idea: buttons underneath the file view, which can toggle
+
