@@ -84,7 +84,7 @@ public:
 
 namespace SongInfoParser
 {
-    struct TrackTextParser;
+    class TrackTextParser;
 }
 
 
@@ -151,6 +151,7 @@ class WebReader : public TagReader
     std::string m_strAlbumName;
     double m_dRating;
     std::string m_strComposer;
+    AlbumInfo::VarArtists m_eVarArtists;
 
     //bool m_bSuppTitle;
     //bool m_bSuppArtist;
@@ -160,8 +161,10 @@ class WebReader : public TagReader
     //bool m_bSuppAlbumName;
     //bool m_bSuppRating;
     bool m_bSuppComposer;
+    bool m_bSuppVarArtists;
 
     std::string m_strType;
+    int convVarArtists() const; // converts m_eVarArtists to an int is either 0 or contains all VA-enabled values, based on configuration
 public:
     /*override*/ std::string getTitle(bool* pbFrameExists = 0) const { if (0 != pbFrameExists) { *pbFrameExists = !m_strTitle.empty(); } return m_strTitle; }
 
@@ -181,6 +184,8 @@ public:
 
     /*override*/ std::string getComposer(bool* pbFrameExists = 0) const { if (0 != pbFrameExists) { *pbFrameExists = !m_strComposer.empty(); } return m_strComposer; }
 
+    /*override*/ int getVariousArtists(bool* pbFrameExists = 0) const { if (0 != pbFrameExists) { *pbFrameExists = AlbumInfo::VA_NOT_SUPP != m_eVarArtists; } return convVarArtists(); }
+
     /*override*/ SuportLevel getSupport(Feature) const;
 
     DECL_RD_NAME("Web");
@@ -226,9 +231,11 @@ public:
     int getOrigPos() const { return m_nOrigPos; }
     const TagReader* getMatchingReader(int i) const; // returns 0 if i is out of range
 
+    void adjustVarArtists(bool b); // if VARIOUS_ARTISTS is not ASSIGNED, sets m_strValue and m_eStatus
+
     struct InvalidValue {};
 private:
-    void reload(); // to be called initially and each time the priority of tag readers changes
+    void setUp(); // to be called initially and each time the priority of tag readers changes
     TagWriter* m_pTagWriter;
     const Mp3Handler* m_pMp3Handler;
     int m_nCrtPos;
@@ -286,6 +293,7 @@ class TagWriter : public QObject
     void sortTagReaders(); // sorts m_vTagReaderInfo so that it matches m_vSortedKnownTagReaders
 
     std::vector<SongInfoParser::TrackTextParser*> m_vpTrackTextParsers; // one TrackTextParser for each pattern
+    std::set<int> m_snActivePatterns;
     std::vector<AlbumInfo> m_vAlbumInfo; // one AlbumInfo for every album data downloaded from structured web sites
 
     // "original value" of a selected field; not necessarily related to the fields in a file's tags, but merely holds whatever happened to be in a given field when it gets selected; the "original value" from Mp3HandlerTagData's point of view is not stored anywhere, but it is recovered as needed, because toggleAssigned() calls "reloadAll("", DONT_CLEAR)"
@@ -325,11 +333,19 @@ class TagWriter : public QObject
     std::set<int> m_snUnassignedImages;
 
     const bool& m_bIsFastSaving;
+    bool m_bShouldShowPatternsNote;
 
     int m_nFileToErase;
 
+    bool m_bVariousArtists;
+    bool m_bAutoVarArtists; // true at first, until the "toggle" button is clicked
+    void adjustVarArtists();
+
+    bool m_bDelayedAdjVarArtists;
+    bool m_bWaitingChangeNotif;
+
 public:
-    TagWriter(CommonData* pCommonData, QWidget* pParentWnd, const bool& bIsFastSaving);
+    TagWriter(CommonData* pCommonData, QWidget* pParentWnd, const bool& bIsFastSaving, const TextCaseOptions& eArtistCase, const TextCaseOptions& eTitleCase);
     ~TagWriter();
 
     enum ClearData { DONT_CLEAR_DATA, CLEAR_DATA };
@@ -369,6 +385,8 @@ public:
     // doesn't throw, but invalid patterns are discarded; it returns false if at least one pattern was discarded;
     bool updatePatterns(const std::vector<std::pair<std::string, int> >&);
     std::vector<std::string> getPatterns() const;
+    void setActivePatterns(const std::set<int>&);
+    std::set<int> getActivePatterns() const { return m_snActivePatterns; }
 
     // model-based nField; UI components should pass the UI index through TagReader::FEATURE_ON_POS[] before calling this
     Mp3HandlerTagData::Status getStatus(int nSong, int nField) const { return m_vpMp3HandlerTagData[nSong]->getStatus(nField); }
@@ -396,6 +414,8 @@ public:
 
     bool isFastSaving() const { return m_bIsFastSaving; }
 
+    bool shouldShowPatternsNote() const { return m_bShouldShowPatternsNote; }
+
     enum ConsiderAssigned { CONSIDER_UNASSIGNED, CONSIDER_ASSIGNED };
     int addImage(const ImageInfo& img, bool bConsiderAssigned); // returns the index of the image; if it already exists it's not added again; if it's invalid returns -1 // "unassigned" images cause warnings when going to another album
     void addImgWidget(ImageInfoPanelWdgImpl*);
@@ -405,20 +425,28 @@ public:
     void clearShowedNonSeqWarn() { m_bShowedNonSeqWarn = false; }
     int getUnassignedImagesCount() const { return int(m_snUnassignedImages.size()); }
 
+    void toggleVarArtists();
+    void delayedAdjVarArtists();
+
+    const TextCaseOptions& m_eArtistCase;
+    const TextCaseOptions& m_eTitleCase;
+
 private slots:
     void onAssignImage(int);
     void onEraseFile(int);
     void onEraseFileDelayed();
+    void onDelayedTrackSeqWarn();
+    void onDelayedAdjVarArtists();
+    void onDelayedChangeNotif();
 
 signals:
     void albumChanged(/*bool bContentOnly*/); // the selection may be kept iff bContentOnly is true
     void fileChanged();
     void imagesChanged();
     void requestSave();
+    void varArtistsUpdated(bool bVarArtists);
 };
 
 
 #endif // #ifndef TagWriterH
 
-
-
