@@ -24,19 +24,21 @@
 #include  <iomanip>
 
 #include  <QDir>
+#include  <QMessageBox>
 
 #include  "Transformation.h"
 
 #include  "Helpers.h"
 #include  "OsFile.h"
 #include  "Mp3Manip.h"
+#include  "CommonData.h"
 
 
 using namespace std;
 using namespace pearl;
 
 
-/*static const string& getOsTempDir() 
+/*static const string& getOsTempDir()
 {
     static string s; // ttt3 these static variables are not really thread safe, but in this case it doesn't matter, because they all get called from a single thread (the UI thread)
     if (s.empty())
@@ -51,7 +53,7 @@ using namespace pearl;
 }
 */
 
-//ttt1 review all defaults
+//ttt2 review all defaults
 static string getDefaultSrc() { return ""; } //ttt2 see if src dir needs to be revived
 
 static string getDefaultUnprocOrig() { return convStr(getTempDir()) + "/mp3diags/unprocOrig"; }
@@ -90,35 +92,98 @@ static string getDefaultComp() { return convStr(getTempDir() + "/mp3diags/comp")
 
 
 
-static TransfConfig::OptionsWrp getDefaultOptions()
+TransfConfig::Options::Options() :
+        //m_nProcOrigChange(5), // move w/o renaming, if doesn't exist; discard if it exists;
+        m_eProcOrigChange(PO_ERASE), // erase
+        m_bProcOrigUseLabel(false),
+        m_bProcOrigAlwayUseCounter(false),
+
+        m_eUnprocOrigChange(UPO_DONT_CHG), // don't change
+        m_bUnprocOrigUseLabel(false),
+        m_bUnprocOrigAlwayUseCounter(false),
+
+        m_eProcessedCreate(PR_CREATE_RENAME_IF_USED), // create and rename if the name is in use
+        m_bProcessedUseLabel(true),
+        m_bProcessedAlwayUseCounter(true),
+        m_bProcessedUseSeparateDir(false),
+
+        m_bTempCreate(false),
+
+        m_bCompCreate(false),
+
+        m_bKeepOrigTime(false)
 {
-    TransfConfig::OptionsWrp x;
-    x.m_nVal = 0;
-
-    x.m_opt.m_bTempCreate = false;
-
-    x.m_opt.m_bCompCreate = false;
-
-    x.m_opt.m_nProcOrigChange = 5; // move w/o renaming, if doesn't exist; discard if it exists;
-    x.m_opt.m_bProcOrigUseLabel = false;
-    x.m_opt.m_bProcOrigAlwayUseCounter = false;
-
-    x.m_opt.m_nUnprocOrigChange = 0; // don't change
-    x.m_opt.m_bUnprocOrigUseLabel = false;
-    x.m_opt.m_bUnprocOrigAlwayUseCounter = false;
-
-    x.m_opt.m_nProcessedCreate = 2; // create and always rename, since it is created in the same dir as the source
-    x.m_opt.m_bProcessedUseLabel = true;
-    x.m_opt.m_bProcessedAlwayUseCounter = true;
-    x.m_opt.m_bProcessedUseSeparateDir = false;
-
-    x.m_opt.m_bKeepOrigTime = false;
-
-    return x;
 }
 
 
 
+
+// returns *this, with some fields changed to match a "backup" configuration; touches many field, but ignores others (m_bProcOrigUseLabel, m_bProcOrigAlwayUseCounter, m_bUnprocOrigUseLabel, m_bUnprocOrigAlwayUseCounter, m_bKeepOrigTime)
+TransfConfig::Options TransfConfig::Options::asBackup() const
+{
+    Options opt (*this);
+
+    opt.m_bTempCreate = false;
+
+    opt.m_bCompCreate = false;
+
+    opt.m_eProcOrigChange = PO_MOVE_OR_ERASE;
+    //opt.m_bProcOrigUseLabel = false;
+    //opt.m_bProcOrigAlwayUseCounter = false;
+
+    opt.m_eUnprocOrigChange = UPO_DONT_CHG; // don't change
+    //opt.m_bUnprocOrigUseLabel = false;
+    //opt.m_bUnprocOrigAlwayUseCounter = false;
+
+    opt.m_eProcessedCreate = PR_CREATE_RENAME_IF_USED; // create and rename if the name is in use
+    opt.m_bProcessedUseLabel = true;
+    opt.m_bProcessedAlwayUseCounter = true;
+    opt.m_bProcessedUseSeparateDir = false;
+
+    //opt.m_bKeepOrigTime = false;
+
+    return opt;
+}
+
+// returns *this, with some fields changed to match a "non-backup" configuration; touches many field, but ignores others (m_bProcOrigUseLabel, m_bProcOrigAlwayUseCounter, m_bUnprocOrigUseLabel, m_bUnprocOrigAlwayUseCounter, m_bKeepOrigTime)
+TransfConfig::Options TransfConfig::Options::asNonBackup() const
+{
+    Options opt (*this);
+
+    opt.m_bTempCreate = false;
+
+    opt.m_bCompCreate = false;
+
+    opt.m_eProcOrigChange = PO_ERASE; // move w/o renaming, if doesn't exist; discard if it exists;
+    //opt.m_bProcOrigUseLabel = false;
+    //opt.m_bProcOrigAlwayUseCounter = false;
+
+    opt.m_eUnprocOrigChange = UPO_DONT_CHG; // don't change
+    //opt.m_bUnprocOrigUseLabel = false;
+    //opt.m_bUnprocOrigAlwayUseCounter = false;
+
+    opt.m_eProcessedCreate = PR_CREATE_RENAME_IF_USED; // create and rename if the name is in use
+    opt.m_bProcessedUseLabel = true;
+    opt.m_bProcessedAlwayUseCounter = true;
+    opt.m_bProcessedUseSeparateDir = false;
+
+    //opt.m_bKeepOrigTime = false;
+
+    return opt;
+}
+
+
+
+// like getDefaultOptions() but does backups rather than erase the original file (if a backup exists, the file is deleted, though)
+/*static* / TransfConfig::Options TransfConfig::getBackupDefaultOptions()
+{
+    Options opt (getDefaultOptions());
+    opt.m_nProcOrigChange = 5; // move w/o renaming, if doesn't exist; discard if it exists;
+    return opt;
+}
+*/
+
+
 TransfConfig::TransfConfig(
         const std::string& strSrcDir,
         const std::string& strProcOrigDir,
@@ -129,6 +194,7 @@ TransfConfig::TransfConfig(
         int nOptions // may be -1 to indicate "default" values
         )
 {
+    m_bInitError = false;
     m_strSrcDir = "*" == strSrcDir ? getDefaultSrc() : strSrcDir;
     m_strProcOrigDir = "*" == strProcOrigDir ? getDefaultProcOrig() : strProcOrigDir;
     m_strUnprocOrigDir = "*" == strUnprocOrigDir ? getDefaultUnprocOrig() : strUnprocOrigDir;
@@ -136,7 +202,17 @@ TransfConfig::TransfConfig(
     m_strTempDir = "*" == strTempDir ? getDefaultTemp() : strTempDir;
     m_strCompDir = "*" == strCompDir ? getDefaultComp() : strCompDir;
 
-    m_optionsWrp.m_nVal = -1 == nOptions ? getDefaultOptions().m_nVal : nOptions;
+    if (nOptions >= 0x00040000)
+    { // only the first 18 bits are supposed to be used; if more seem to be used, it is because of a manually entered wrong value or because a change in the bitfield representation;
+        QMessageBox::critical(getMainForm(), "Error", "Invalid value found for file settings. Reverting to default settings.");
+        nOptions = -1;
+        m_bInitError = true;
+    }
+
+    if (-1 != nOptions)
+    {
+        m_options.setVal(nOptions);
+    }
 
     checkDirName(m_strSrcDir);
     checkDirName(m_strProcOrigDir);  // some of these dirs are allowed to be empty, just like src when comes from the config dialog, empty meaning "/", so that should work for all, probably
@@ -155,8 +231,6 @@ TransfConfig::TransfConfig()
     m_strProcessedDir = getDefaultProcessed();
     m_strTempDir = getDefaultTemp();
     m_strCompDir = getDefaultComp();
-
-    m_optionsWrp.m_nVal = getDefaultOptions().m_nVal;
 }
 
 
@@ -171,7 +245,7 @@ void TransfConfig::splitOrigName(const string& strOrigSrcName, std::string& strR
 #ifndef WIN32
     CB_CHECK1 (beginsWith(strRelDir, getPathSepAsStr()), IncorrectPath());
 #else
-    //CB_CHECK1 (beginsWith(strRelDir, getPathSepAsStr()), IncorrectPath()); //ttt1 see if it's OK to skip this test on Windows; it probably is, because it's more of an assert and the fact that it's not triggered on Linux should be enough
+    //CB_CHECK1 (beginsWith(strRelDir, getPathSepAsStr()), IncorrectPath()); //ttt2 see if it's OK to skip this test on Windows; it probably is, because it's more of an assert and the fact that it's not triggered on Linux should be enough
 #endif
 
 
@@ -290,27 +364,27 @@ string TransfConfig::getRenamedName(const std::string& strOrigSrcName, const std
 
 TransfConfig::OrigFile TransfConfig::getChangedOrigNameHlp(const string& strOrigSrcName, const string& strDestDir, int nChange, bool bUseLabel, bool bAlwayUseCounter, string& strNewName) const
 {
-    switch (nChange)
+    switch (nChange) // !!! nChange must match both ProcOrig and UnprocOrig, since getChangedOrigNameHlp() gets called in both cases
     {
-    case 0: // don't change unprocessed orig file
+    case Options::PO_DONT_CHG: // don't change unprocessed orig file
         strNewName = strOrigSrcName; return ORIG_DONT_CHANGE;
 
-    case 1: // erase unprocessed orig file
+    case Options::PO_ERASE: // erase unprocessed orig file
         strNewName = strOrigSrcName; return ORIG_ERASE;
 
-    case 2: // move unprocessed orig file to strDestDir; always rename
+    case Options::PO_MOVE_ALWAYS_RENAME: // move unprocessed orig file to strDestDir; always rename
         strNewName = getRenamedName(strOrigSrcName, strDestDir, bUseLabel ? "orig" : "", bAlwayUseCounter, ALWAYS_RENAME);
         return ORIG_MOVE;
 
-    case 3: // move unprocessed orig file to strDestDir; rename only if name is in use
+    case Options::PO_MOVE_RENAME_IF_USED: // move unprocessed orig file to strDestDir; rename only if name is in use
         strNewName = getRenamedName(strOrigSrcName, strDestDir, bUseLabel ? "orig" : "", bAlwayUseCounter, RENAME_IF_NEEDED);
         return ORIG_MOVE;
 
-    case 4: // rename in the same dir
+    case Options::PO_RENAME_SAME_DIR: // rename in the same dir
         strNewName = getRenamedName(strOrigSrcName, m_strSrcDir, bUseLabel ? "orig" : "", bAlwayUseCounter, ALWAYS_RENAME);
         return ORIG_MOVE;
 
-    case 5: // move unprocessed orig file to strDestDir if it's not already there; erase if it exists; don't
+    case Options::PO_MOVE_OR_ERASE: // move unprocessed orig file to strDestDir if it's not already there; erase if it exists;
         strNewName = getRenamedName(strOrigSrcName, strDestDir, "", USE_COUNTER_IF_NEEDED, RENAME_IF_NEEDED, ALLOW_DUPLICATES);
         return ORIG_MOVE_OR_ERASE;
 
@@ -323,14 +397,14 @@ TransfConfig::OrigFile TransfConfig::getChangedOrigNameHlp(const string& strOrig
 TransfConfig::OrigFile TransfConfig::getProcOrigName(string strOrigSrcName, string& strNewName) const
 {
     removeSuffix(strOrigSrcName);
-    return getChangedOrigNameHlp(strOrigSrcName, m_strProcOrigDir, m_optionsWrp.m_opt.m_nProcOrigChange, m_optionsWrp.m_opt.m_bProcOrigUseLabel, m_optionsWrp.m_opt.m_bProcOrigAlwayUseCounter, strNewName);
+    return getChangedOrigNameHlp(strOrigSrcName, m_strProcOrigDir, m_options.m_eProcOrigChange, m_options.m_bProcOrigUseLabel, m_options.m_bProcOrigAlwayUseCounter, strNewName);
 }
 
 
 TransfConfig::OrigFile TransfConfig::getUnprocOrigName(string strOrigSrcName, string& strNewName) const
 {
     removeSuffix(strOrigSrcName);
-    return getChangedOrigNameHlp(strOrigSrcName, m_strUnprocOrigDir, m_optionsWrp.m_opt.m_nUnprocOrigChange, m_optionsWrp.m_opt.m_bUnprocOrigUseLabel, m_optionsWrp.m_opt.m_bUnprocOrigAlwayUseCounter, strNewName);
+    return getChangedOrigNameHlp(strOrigSrcName, m_strUnprocOrigDir, m_options.m_eUnprocOrigChange, m_options.m_bUnprocOrigUseLabel, m_options.m_bUnprocOrigAlwayUseCounter, strNewName);
 }
 
 
@@ -433,17 +507,17 @@ void TransfConfig::testRemoveSuffix() const
 TransfConfig::TransfFile TransfConfig::getProcessedName(string strOrigSrcName, std::string& strName) const
 {
     removeSuffix(strOrigSrcName);
-    switch (m_optionsWrp.m_opt.m_nProcessedCreate)
+    switch (m_options.m_eProcessedCreate)
     {
-    case 0: // don't create proc files
+    case Options::PR_DONT_CREATE: // don't create proc files
         strName.clear(); return TRANSF_DONT_CREATE;
 
-    case 1: // create proc files and always rename
-        strName = getRenamedName(strOrigSrcName, m_optionsWrp.m_opt.m_bProcessedUseSeparateDir ? m_strProcessedDir : m_strSrcDir, m_optionsWrp.m_opt.m_bProcessedUseLabel ? "proc" : "", m_optionsWrp.m_opt.m_bProcessedAlwayUseCounter, ALWAYS_RENAME);
+    case Options::PR_CREATE_ALWAYS_RENAME: // create proc files and always rename
+        strName = getRenamedName(strOrigSrcName, m_options.m_bProcessedUseSeparateDir ? m_strProcessedDir : m_strSrcDir, m_options.m_bProcessedUseLabel ? "proc" : "", m_options.m_bProcessedAlwayUseCounter, ALWAYS_RENAME);
         return TRANSF_CREATE;
 
-    case 2: // create proc files and rename if the name is in use
-        strName = getRenamedName(strOrigSrcName, m_optionsWrp.m_opt.m_bProcessedUseSeparateDir ? m_strProcessedDir : m_strSrcDir, m_optionsWrp.m_opt.m_bProcessedUseLabel ? "proc" : "", m_optionsWrp.m_opt.m_bProcessedAlwayUseCounter, RENAME_IF_NEEDED);
+    case Options::PR_CREATE_RENAME_IF_USED: // create proc files and rename if the name is in use
+        strName = getRenamedName(strOrigSrcName, m_options.m_bProcessedUseSeparateDir ? m_strProcessedDir : m_strSrcDir, m_options.m_bProcessedUseLabel ? "proc" : "", m_options.m_bProcessedAlwayUseCounter, RENAME_IF_NEEDED);
         return TRANSF_CREATE;
 
     default:
@@ -455,7 +529,7 @@ TransfConfig::TransfFile TransfConfig::getProcessedName(string strOrigSrcName, s
 TransfConfig::TransfFile TransfConfig::getTempName(string strOrigSrcName, const std::string& strOpName, std::string& strName) const
 {
     removeSuffix(strOrigSrcName);
-    if (!m_optionsWrp.m_opt.m_bTempCreate)
+    if (!m_options.m_bTempCreate)
     {
         strName = getTempFile(strOrigSrcName);
         return TRANSF_DONT_CREATE;
@@ -481,7 +555,7 @@ TransfConfig::TransfFile TransfConfig::getCompNames(string strOrigSrcName, const
 {
     removeSuffix(strOrigSrcName);
 
-    if (!m_optionsWrp.m_opt.m_bCompCreate)
+    if (!m_options.m_bCompCreate)
     {
         strBefore = getTempFile(strOrigSrcName);
         strAfter = getTempFile(strOrigSrcName);
@@ -575,20 +649,20 @@ TransfConfig::TransfFile TransfConfig::getCompNames(string strOrigSrcName, const
 
 TransfConfig::OrigFile TransfConfig::getProcOrigAction() const
 {
-    switch (m_optionsWrp.m_opt.m_nProcOrigChange)
+    switch (m_options.m_eProcOrigChange)
     {
-    case 0: // don't change unprocessed orig file
+    case Options::PO_DONT_CHG: // don't change unprocessed orig file
         return ORIG_DONT_CHANGE;
 
-    case 1: // erase unprocessed orig file
+    case Options::PO_ERASE: // erase unprocessed orig file
         return ORIG_ERASE;
 
-    case 2: // move unprocessed orig file to strDestDir; always rename
-    case 3: // move unprocessed orig file to strDestDir; rename only if name is in use
-    case 4: // rename in the same dir
+    case Options::PO_MOVE_ALWAYS_RENAME: // move unprocessed orig file to strDestDir; always rename
+    case Options::PO_MOVE_RENAME_IF_USED: // move unprocessed orig file to strDestDir; rename only if name is in use
+    case Options::PO_RENAME_SAME_DIR: // rename in the same dir
         return ORIG_MOVE;
 
-    case 5: // move if dest doesn't exist; erase if it does;
+    case Options::PO_MOVE_OR_ERASE: // move if dest doesn't exist; erase if it does;
         return ORIG_MOVE_OR_ERASE;
 
     default:
@@ -596,19 +670,20 @@ TransfConfig::OrigFile TransfConfig::getProcOrigAction() const
     }
 }
 
+
 TransfConfig::OrigFile TransfConfig::getUnprocOrigAction() const
 {
-    switch (m_optionsWrp.m_opt.m_nUnprocOrigChange)
+    switch (m_options.m_eUnprocOrigChange)
     {
-    case 0: // don't change unprocessed orig file
+    case Options::UPO_DONT_CHG: // don't change unprocessed orig file
         return ORIG_DONT_CHANGE;
 
-    case 1: // erase unprocessed orig file
+    case Options::UPO_ERASE: // erase unprocessed orig file
         return ORIG_ERASE;
 
-    case 2: // move unprocessed orig file to strDestDir; always rename
-    case 3: // move unprocessed orig file to strDestDir; rename only if name is in use
-    case 4: // rename in the same dir
+    case Options::UPO_MOVE_ALWAYS_RENAME: // move unprocessed orig file to strDestDir; always rename
+    case Options::UPO_MOVE_RENAME_IF_USED: // move unprocessed orig file to strDestDir; rename only if name is in use
+    case Options::UPO_RENAME_SAME_DIR: // rename in the same dir
         return ORIG_MOVE;
 
     default:
@@ -619,13 +694,13 @@ TransfConfig::OrigFile TransfConfig::getUnprocOrigAction() const
 
 TransfConfig::TransfFile TransfConfig::getProcessedAction() const
 {
-    switch (m_optionsWrp.m_opt.m_nProcessedCreate)
+    switch (m_options.m_eProcessedCreate)
     {
-    case 0: // don't create proc files
+    case Options::PR_DONT_CREATE: // don't create proc files
         return TRANSF_DONT_CREATE;
 
-    case 1: // create proc files and always rename
-    case 2: // create proc files and rename if the name is in use
+    case Options::PR_CREATE_ALWAYS_RENAME: // create proc files and always rename
+    case Options::PR_CREATE_RENAME_IF_USED: // create proc files and rename if the name is in use
         return TRANSF_CREATE;
 
     default:
@@ -635,12 +710,12 @@ TransfConfig::TransfFile TransfConfig::getProcessedAction() const
 
 TransfConfig::TransfFile TransfConfig::getTempAction() const
 {
-    return m_optionsWrp.m_opt.m_bTempCreate ? TRANSF_CREATE : TRANSF_DONT_CREATE;
+    return m_options.m_bTempCreate ? TRANSF_CREATE : TRANSF_DONT_CREATE;
 }
 
 TransfConfig::TransfFile TransfConfig::getCompAction() const
 {
-    return m_optionsWrp.m_opt.m_bCompCreate ? TRANSF_CREATE : TRANSF_DONT_CREATE;
+    return m_options.m_bCompCreate ? TRANSF_CREATE : TRANSF_DONT_CREATE;
 }
 
 
@@ -741,14 +816,14 @@ std::string TransfConfig::getProcName(const std::string& strOrigSrcName) const
     return CHANGED_NO_RECALL;
 }
 
-//ttt1 test transformation (and perhaps the other threads) with this: wheat happens if something takes very long and the user presses both pause and resume before the UI checks
+//ttt2 test transformation (and perhaps the other threads) with this: wheat happens if something takes very long and the user presses both pause and resume before the UI checks
 
 
 
 
 /*
 
-ttt1 warnings + info
+ttt2 warnings + info
 
 
 ID3 V2.3
@@ -780,11 +855,12 @@ ttt2 When removing frames and Xing/Lame is present, adjust header for count and
 //ttt2 perhaps separate tab with actions: "check quality", ... ; or perhaps tab with settings, so the quality is checked the first time
 
 //ttt2 there are both "fixes" (pad truncated ...) and "actions" ("remove lyrics", "add picture", "check ID3"); perhaps distinguish between them
-//ttt1 "check tags" command - goes to musicbrainz and checks all known tags; 2009.03.19 - this doesn't seem to make much sense, given that there are many albums with the same name, ... ; perhaps we should search by track duration too;
+//ttt2 "check tags" command - goes to musicbrainz and checks all known tags; 2009.03.19 - this doesn't seem to make much sense, given that there are many albums with the same name, ... ; perhaps we should search by track duration too;
 
 
 
-//ttt1 transform: convert to id3v2: assumes whatever reader order is currently in tag editor and writes to id3v2 if there are differences (including rating); maybe option to convert ID3V2.4.0 reagrdless; doing it manually isn't such a big pain, though;
+//ttt2 transform: convert to id3v2: assumes whatever reader order is currently in tag editor and writes to id3v2 if there are differences (including rating); maybe option to convert ID3V2.4.0 reagrdless; doing it manually isn't such a big pain, though;
 
 
+//ttt1 see why some files are read-only on W7
 
