@@ -57,18 +57,21 @@ std::string, without the need of calling c_str().
 
 */
 
-
+#ifdef __GNUC__
 
 #include  <ext/stdio_filebuf.h>
 #include  <istream>
 #include  <ostream>
+#include  <fstream>
 
 #include  <fcntl.h> // for open()
 
 
 template<class T>
-int unicodeOpenHlp(T handler, std::ios_base::openmode __mode);
+int unicodeOpenHlp(const T& handler, std::ios_base::openmode __mode);
 
+// converts __mode to flags that can be used by the POSIX open() function
+int getOpenFlags(std::ios_base::openmode __mode);
 
 
 
@@ -261,7 +264,7 @@ public:
       */
       template<class T>
       void
-      open(T x, std::ios_base::openmode __mode = std::ios_base::in)
+      open(const T& x, std::ios_base::openmode __mode = std::ios_base::in)
       {
         if (!_M_filebuf.open(unicodeOpenHlp(x, __mode | std::ios_base::in), __mode | std::ios_base::in))
           this->setstate(std::ios_base::failbit);
@@ -393,7 +396,7 @@ public:
       */
       template<class T>
       void
-      open(T x,
+      open(const T& x,
            std::ios_base::openmode __mode = std::ios_base::out | std::ios_base::trunc)
       {
         if (!_M_filebuf.open(unicodeOpenHlp(x, __mode | std::ios_base::out), __mode | std::ios_base::out))
@@ -525,7 +528,7 @@ public:
       */
       template<class T>
       void
-      open(T x,
+      open(const T& x,
            std::ios_base::openmode __mode = std::ios_base::in | std::ios_base::out)
       {
         if (!_M_filebuf.open(unicodeOpenHlp(x, __mode), __mode))
@@ -575,5 +578,121 @@ typedef basic_ofstream_unicode<char> ofstream_unicode;
 typedef basic_fstream_unicode<char> fstream_unicode;
 
 
-#endif
+#elif defined(_MSC_VER) && _MSC_VER>=1400 // #ifdef __GNUC__
 
+// Visual Studio port by Sebastian Schuberth
+
+// As of Visual Studio 2005 (aka version 8.0), the supplied STL has extensions
+// that accept filenames of type wchar_t.
+
+#include <fstream>
+
+#define NOMINMAX 1
+#define WIN32_LEAN_AND_MEAN 1
+#include <windows.h>
+
+typedef std::ifstream ifstream_unicode;
+typedef std::ofstream ofstream_unicode;
+typedef std::fstream fstream_unicode;
+
+class ifstream_utf8:public std::ifstream
+{
+  public:
+
+    explicit ifstream_utf8(char const* _Filename,ios_base::openmode _Mode=ios_base::in,int _Prot=(int)ios_base::_Openprot)
+    {
+        open(_Filename,_Mode,_Prot);
+    }
+
+    void open(char const* _Filename,ios_base::openmode _Mode=ios_base::in,int _Prot=(int)ios_base::_Openprot)
+    {
+        int length=MultiByteToWideChar(CP_UTF8,0,_Filename,-1,NULL,0);
+        if (length>0) {
+            wchar_t* buffer=new wchar_t[length+1];
+            MultiByteToWideChar(CP_UTF8,0,_Filename,-1,buffer,length);
+            std::ifstream::open(buffer,_Mode,_Prot);
+            delete [] buffer;
+        }
+        else {
+            setstate(ios_base::failbit);
+        }
+    }
+
+    void open(char const* _Filename,ios_base::open_mode _Mode)
+    {
+        open(_Filename,_Mode);
+    }
+};
+
+class ofstream_utf8:public std::ofstream
+{
+  public:
+
+    explicit ofstream_utf8(char const* _Filename,ios_base::openmode _Mode=ios_base::out,int _Prot=(int)ios_base::_Openprot)
+    {
+        open(_Filename,_Mode,_Prot);
+    }
+
+    void open(char const* _Filename,ios_base::openmode _Mode=ios_base::out,int _Prot=(int)ios_base::_Openprot)
+    {
+        int length=MultiByteToWideChar(CP_UTF8,0,_Filename,-1,NULL,0);
+        if (length>0) {
+            wchar_t* buffer=new wchar_t[length+1];
+            MultiByteToWideChar(CP_UTF8,0,_Filename,-1,buffer,length);
+            std::ofstream::open(buffer,_Mode,_Prot);
+            delete [] buffer;
+        }
+        else {
+            setstate(ios_base::failbit);
+        }
+    }
+
+    void open(char const* _Filename,ios_base::open_mode _Mode)
+    {
+        open(_Filename,_Mode);
+    }
+};
+
+class fstream_utf8:public std::fstream
+{
+  public:
+
+    explicit fstream_utf8(char const* _Filename,ios_base::openmode _Mode=ios_base::in|ios_base::out,int _Prot=(int)ios_base::_Openprot)
+    {
+        open(_Filename,_Mode,_Prot);
+    }
+
+    void open(char const* _Filename,ios_base::openmode _Mode=ios_base::in|ios_base::out,int _Prot=(int)ios_base::_Openprot)
+    {
+        int length=MultiByteToWideChar(CP_UTF8,0,_Filename,-1,NULL,0);
+        if (length>0) {
+            wchar_t* buffer=new wchar_t[length+1];
+            MultiByteToWideChar(CP_UTF8,0,_Filename,-1,buffer,length);
+            std::fstream::open(buffer,_Mode,_Prot);
+            delete [] buffer;
+        }
+        else {
+            setstate(ios_base::failbit);
+        }
+    }
+
+    void open(char const* _Filename,ios_base::open_mode _Mode)
+    {
+        open(_Filename,_Mode);
+    }
+};
+
+#else // _MSC_VER / __GNUC__
+
+    //#error classes i/ofstream_utf8 need to be ported to this compiler
+    #warning classes i/ofstream_utf8 need to be ported to your compiler
+
+    #include  <fstream>
+
+    typedef std::basic_ifstream<char> ifstream_utf8;
+    typedef std::basic_ofstream<char> ofstream_utf8;
+    typedef std::basic_fstream<char> fstream_utf8;
+
+#endif // _MSC_VER / __GNUC__
+
+#endif // FStreamUtf8H
